#include <fork.hpp>
#include <join.hpp>
#include <root.hpp>
#include <cstdio>


using namespace typon::rt;

int fibo(int n) {
  if (n < 2) {
    return n;
  }
  return fibo(n - 1) + fibo(n - 2);
}

Fork<int> fib() {
  co_return fibo(30);
}

Fork<void> fail() {
  throw std::exception();
  co_return;
}

Join<void> join(int max) {
  std::vector<Future<Fork<int>::promise_type>> v;
  for (int i = 0; i < max; i++) {
    v.push_back(co_await fib());
  }
  co_await fail();
  co_await Sync();
  for (auto & f : v) {
    printf("result = %d\n", f.get());
  }
  puts("info: join exited normally");
}

Root root1() {
  try {
    co_await join(0);
    puts("error: join did not raise an exception");
  } catch(std::exception &) {
    puts("ok: join raised an exception");
  }
}

Root root2() {
  try {
    co_await join(20);
    puts("error: join did not raise an exception");
  } catch(std::exception &) {
    puts("ok: join raised an exception");
  }
}

int main() {
  puts("join(0)");
  root1().call();
  puts("join(20)");
  root2().call();
  puts("done");
  return 0;
}

