#include <fork.hpp>
#include <join.hpp>
#include <root.hpp>
#include <task.hpp>
#include <cstdio>


using namespace typon::rt;

Task<int> fibo(int n) {
  // printf("fibo(%d)\n", n);
  if (n < 2) {
    co_return n;
  }
  int a = co_await fibo(n - 1);
  int b = co_await fibo(n - 2);
  co_return a + b;
}

Join<int> parallel_fibo(int n);
Fork<int> fork_fibo(int n);

Join<int> parallel_fibo(int n) {
  // printf("parallel_fibo(%d)\n", n);
  if (n < 2) {
    co_return n;
  }
  if (n < 10)
  {
    int a = co_await fibo(n - 1);
    int b = co_await fibo(n - 2);
    // printf("// parallel_fibo(%d)\n", n);
    co_return a + b;
  }
  Future a = co_await fork_fibo(n - 1);
  Future b = co_await fork_fibo(n - 2);
  co_await Sync();
  co_return a.get() + b.get();
}

Fork<int> fork_fibo(int n) {
  co_return co_await parallel_fibo(n);
}

Root root() {
  int result = co_await parallel_fibo(40);
  printf("result = %d\n", result);
}

int main() {
  root().call();
  puts("done");
  return 0;
}
